# /**
#
# \name{detectReplicas}
#
# \title{detectReplicas}
#
# \alias{detectReplicas}
#
# \keyword{utilities}
#
# @usage
#
# \description{The function returns for each unique item all corresponding indexes.}
#
# \arguments{
#  \item{arrayDescription}{an object of class \code{data.frame}
#                          which contains a column named as
#                          \code{spotIdentifier}; required; default missing
#                          }
#  \item{spotIdentifier}{character string; required; default: "ID"}
#  \item{identifiersToBeSkipped}{vector of character strings;
#                                required; default: "Blank";
#                                items of the column \code{spotIdentifier}
#                                for which no replica detection takes place}
# }
#
# \value{A list which contains \code{nrOfReplicas} and
#        \code{spotReplicas}.  \code{nrOfReplicas}: one
#        integer characterizing the number of spot replicas
#        given for each identifier if existing or otherwise  \code{NA}.
#        \code{spotReplicas}: a list of the length of the unique
#        identifiers where each element contains a vector of
#        indexes corresponding to the given identifier (i.e. the
#        name of the list element) otherwise \code{NA}.}
#
# \examples{
#
#aD <- data.frame(ID=c("z", "x", "x", "x", "y", "z", "z", "y", "y"))
#re <- detectReplicas(aD, identifiersToBeSkipped = c("Blank", "Control1", "Control2"))
#stopifnot(re[["nrOfReplicas"]] == 3 )
#
#aD <- data.frame(ID=c("Blank", "Control1", "Blank", "Control2"))
#re <- detectReplicas(aD, identifiersToBeSkipped = c("Blank", "Control1", "Control2"))
#stopifnot(is.na(re[["nrOfReplicas"]]))
#
# \dontshow{
#aD <- data.frame(ID=c("Blank", "Blank", "x", "x", "Control1", "y", "x", "z", "y", "Control2"))
#re <- detectReplicas(aD, identifiersToBeSkipped = c("Blank", "Control1", "Control2"))
#stopifnot(is.na(re[["nrOfReplicas"]]))
#
#aD <- data.frame(ID=c("z", "x", "Control1", "y"))
#re <- detectReplicas(aD, identifiersToBeSkipped = c("Control1"))
#stopifnot(re[["nrOfReplicas"]] == 1 )
#
#aD <- data.frame(ID=c("Blank", "x", "x", "Control1", "y", "z", "z", "y", "Control2"))
#re <- detectReplicas(aD, identifiersToBeSkipped = c("Blank", "Control1", "Control2"))
#stopifnot(re[["nrOfReplicas"]] == 2 )
# }
#  }
#
# \author{Andreas Buness <a.buness@dkfz.de>}
#
# */
detectReplicas <- function(arrayDescription,
                           spotIdentifier = "ID",
                           identifiersToBeSkipped = "Blank"
                           ){

  if( missing( arrayDescription ) ){
    stop(" arrayDescription is missing in detectReplicas ")
  }
  if( !is.data.frame(arrayDescription) ){
    stop(" arrayDescription has to be of class data.frame in detectReplicas ")
  } 

  
  # initialization
  nrOfReplicas <- NA
  spotReplicas <- NA
  
  if(is.null(arrayDescription[,spotIdentifier])){
    tmpString <- paste("spotIdentifier",spotIdentifier," is null in data.frame arrayDescription")
    cat(tmpString)
    warning(tmpString)
    return(list(nrOfReplicas=nrOfReplicas,spotReplicas=spotReplicas))
  }

  nonValidItems <- sapply(identifiersToBeSkipped, function(x) return(which( arrayDescription[,spotIdentifier] == x)), simplify=TRUE )
  nonValidItems <- na.exclude(as.numeric(unlist(nonValidItems)))

  if( length(nonValidItems) == 0 ){
    spotIDs <- arrayDescription[,spotIdentifier]
  }else{
    spotIDs <- arrayDescription[,spotIdentifier][-nonValidItems]
  }

  if(length(spotIDs) > 0 ){
    spotReplicas <- lapply(unique(spotIDs), function(x) return(which( arrayDescription[,spotIdentifier] == x ))) 
    names(spotReplicas) <- unique(spotIDs)

    distributionOfReplicas <- sapply(spotReplicas, length)
    maxReplicas <-  max(distributionOfReplicas)
    minReplicas <-  min(distributionOfReplicas)
    if( maxReplicas == minReplicas ){
      nrOfReplicas <- maxReplicas
      cat("\n every item is spotted ");cat(nrOfReplicas);cat(" times \n") 
    }else{
      cat("\n no single/duplicate/triplcate ... spotting has been detected \n")
      cat(paste(" the number of spot replicas varies between ", minReplicas, " and ", maxReplicas, "\n",sep=""))
      cat(" the distribution is given by\n")
      cat(" (first line type, e.g. duplicate = 2, triplicate= 3 \n")
      cat("   second line number of cases )\n")
      distTable <- table(factor(distributionOfReplicas))
      cat(format(dimnames(distTable)));cat("\n")
      cat(distTable);cat("\n")
    }
  }else{
    cat(" no spot identifiers given\n")
  }
    
  return(list(nrOfReplicas=nrOfReplicas,spotReplicas=spotReplicas))
  
}

